@@ -9,16 +9,29 @@ g.wavread = function(wavfile,start=1,end=100,units="minutes") {
   #-------------------------------------------------------
   # extract info from header: fileEncoding does not seem to be consistent, so try variants
   header = c()
-  try(expr={header = rownames(read.csv(wavfile,nrow=15,header=TRUE))},silent=TRUE)
-  if (length(header) == 0) {
-    header = rownames(read.csv(wavfile,skipNul=TRUE,nrow=15,header=TRUE,fileEncoding="WINDOWS-1252"))
-  }
-  if (length(header) == 0) {
-    header = rownames(read.csv(wavfile,skipNul=TRUE,nrow=15,header=TRUE,fileEncoding="UTF-8"))
-  }
-  if (length(header) == 0) {
-    header = rownames(read.csv(wavfile,skipNul=TRUE,nrow=15,header=TRUE,fileEncoding="latin1"))
+  Nlines = 16
+  while (length(header) == 0 | length(grep("Scale-3",header)) == 0 |
+         length(grep("Scale-2",header)) == 0 |
+         length(grep("Scale-1",header)) == 0) { # as we do not know what header size is, search for it (needed in R version =< 3.1)
+    
+    try(expr={header = rownames(read.csv(wavfile,nrow=Nlines,header=TRUE))},silent=TRUE)
+    if (length(header) == 0) {
+      header = rownames(read.csv(wavfile,skipNul=TRUE,nrow=Nlines,header=TRUE,fileEncoding="WINDOWS-1252"))
+    }
+    if (length(header) == 0) {
+      header = rownames(read.csv(wavfile,skipNul=TRUE,nrow=Nlines,header=TRUE,fileEncoding="UTF-8"))
+    }
+    if (length(header) == 0) {
+      header = rownames(read.csv(wavfile,skipNul=TRUE,nrow=Nlines,header=TRUE,fileEncoding="latin1"))
+    }
+    
+    Nlines = Nlines - 1
+    if (Nlines == 10) print("Error: wav file header not recognized in function g.wavread")
   }
+  scale3position = grep("Scale-3",header)
+  scale2position = grep("Scale-2",header)
+  scale1position = grep("Scale-1",header)
+  header = header[1:scale3position]
   P = sapply(as.character(header),function(x) {
     tmp = unlist(strsplit(x,": "))
     if (length(tmp) == 1) {
@@ -52,4 +65,4 @@ g.wavread = function(wavfile,start=1,end=100,units="minutes") {
   # g = (B$C4/ range) # temperature, light and battery ?
   invisible(list(rawxyz=rawxyz,header=P,timestamp = timestamp))
   
-}
\ No newline at end of file
+}
